@@ -266,11 +266,9 @@ from mercurial import (
     help,
     hg,
     lock as lockmod,
-    logcmdutil,
-    node,
+    node as nodemod,
     pycompat,
     revset,
-    scmutil,
 )
 
 from mercurial.i18n import _
@@ -350,7 +348,6 @@ templatekeyword = eh.templatekeyword
 # Configuration
 eh.configitem(b'experimental', b'evolutioncommands', [])
 eh.configitem(b'experimental', b'evolution.allnewcommands', None)
-eh.configitem(b'experimental', b'prunestrip', False)
 
 #####################################################################
 ### Option configuration                                          ###
@@ -434,7 +431,7 @@ def setupparentcommand(ui):
     @eh.command(b'pstatus', pstatusopts,
                 **compat.helpcategorykwargs('CATEGORY_WORKING_DIRECTORY'))
     def pstatus(ui, repo, *args, **kwargs):
-        """show status combining committed and uncommited changes
+        """show status combining committed and uncommitted changes
 
         This show the combined status of the current working copy parent commit and
         the uncommitted change in the working copy itself. The status displayed
@@ -450,7 +447,7 @@ def setupparentcommand(ui):
     @eh.command(b'pdiff', pdiffopts,
                 **compat.helpcategorykwargs('CATEGORY_WORKING_DIRECTORY'))
     def pdiff(ui, repo, *args, **kwargs):
-        """show diff combining committed and uncommited changes
+        """show diff combining committed and uncommitted changes
 
         This show the combined diff of the current working copy parent commit and
         the uncommitted change in the working copy itself. The diff displayed
@@ -655,7 +652,7 @@ def _warnobsoletewc(ui, repo, prevnode=None, wasobs=None):
     if rev.node() == prevnode and wasobs:
         return
     msg = _(b"working directory parent is obsolete! (%s)\n")
-    shortnode = node.short(rev.node())
+    shortnode = nodemod.short(rev.node())
 
     ui.warn(msg % shortnode)
 
@@ -758,7 +755,7 @@ def _installimportobsolete(ui):
 def _getnodefrompatch(patch, dest):
     patchnode = patch.get(b'nodeid')
     if patchnode is not None:
-        dest[b'node'] = node.bin(patchnode)
+        dest[b'node'] = nodemod.bin(patchnode)
 
 @eh.wrapfunction(mercurial.cmdutil, 'tryimportone')
 def tryimportone(orig, ui, repo, hunk, parents, opts, *args, **kwargs):
@@ -831,7 +828,7 @@ def _gettopicidx(ctx):
 def _getcurrenttopic(repo):
     return getattr(repo, 'currenttopic', b'')
 
-def _prevupdate(repo, displayer, target, bookmark, dryrun, mergeopt):
+def _prevupdate(repo, display, target, bookmark, dryrun, mergeopt):
     if dryrun:
         repo.ui.write(_(b'hg update %s;\n') % target)
         if bookmark is not None:
@@ -865,9 +862,9 @@ def _prevupdate(repo, displayer, target, bookmark, dryrun, mergeopt):
                 lockmod.release(tr, lock)
 
     if not repo.ui.quiet:
-        displayer.show(target)
+        display(target)
 
-def _findprevtarget(repo, displayer, movebookmark=False, topic=True):
+def _findprevtarget(repo, display, movebookmark=False, topic=True):
     target = bookmark = None
     wkctx = repo[None]
     p1 = wkctx.p1()
@@ -881,7 +878,7 @@ def _findprevtarget(repo, displayer, movebookmark=False, topic=True):
         parents = [ctx for ctx in parents if ctx.topic() == currenttopic]
 
     # issue message for the various case
-    if p1.node() == node.nullid:
+    if p1.node() == nullid:
         repo.ui.warn(_(b'already at repository root\n'))
     elif not parents and currenttopic:
         repo.ui.warn(_(b'no parent in topic "%s"\n') % currenttopic)
@@ -897,7 +894,7 @@ def _findprevtarget(repo, displayer, movebookmark=False, topic=True):
         choosedrev = utility.revselectionprompt(repo.ui, repo, prevs, header)
         if choosedrev is None:
             for p in parents:
-                displayer.show(p)
+                display(p)
             repo.ui.warn(_(b'multiple parents, explicitly update to one\n'))
         else:
             target = repo[choosedrev]
@@ -942,10 +939,10 @@ def cmdprevious(ui, repo, **opts):
         if topic and hastopic:
             template = utility.stacktemplate
 
-        displayer = logcmdutil.changesetdisplayer(ui, repo,
-                                                  {b'template': template})
+        display = compat.format_changeset_summary_fn(ui, repo, b'previous',
+                                                     template)
 
-        target, bookmark = _findprevtarget(repo, displayer,
+        target, bookmark = _findprevtarget(repo, display,
                                            opts.get('move_bookmark'), topic)
         if target is not None:
             configoverride = util.nullcontextmanager()
@@ -954,7 +951,7 @@ def cmdprevious(ui, repo, **opts):
                     (b'_internal', b'keep-topic'): b'yes'
                 }, source=b'topic-extension')
             with configoverride:
-                _prevupdate(repo, displayer, target, bookmark, dryrunopt,
+                _prevupdate(repo, display, target, bookmark, dryrunopt,
                             mergeopt)
             return 0
         else:
@@ -1001,8 +998,8 @@ def cmdnext(ui, repo, **opts):
             children = [ctx for ctx in children if ctx not in filtered]
             template = utility.stacktemplate
             opts['stacktemplate'] = True
-        displayer = logcmdutil.changesetdisplayer(ui, repo,
-                                                  {b'template': template})
+        display = compat.format_changeset_summary_fn(ui, repo, b'next',
+                                                     template)
 
         # check if we need to evolve while updating to the next child revision
         needevolve = False
@@ -1038,7 +1035,7 @@ def cmdnext(ui, repo, **opts):
 
         if len(children) == 1:
             c = children[0]
-            return _updatetonext(ui, repo, c, displayer, opts)
+            return _updatetonext(ui, repo, c, display, opts)
         elif children:
             cheader = _(b"ambiguous next changeset, choose one to update:")
             crevs = [c.rev() for c in children]
@@ -1046,11 +1043,11 @@ def cmdnext(ui, repo, **opts):
             if choosedrev is None:
                 ui.warn(_(b"ambiguous next changeset:\n"))
                 for c in children:
-                    displayer.show(c)
+                    display(c)
                 ui.warn(_(b"explicitly update to one of them\n"))
                 return 1
             else:
-                return _updatetonext(ui, repo, repo[choosedrev], displayer, opts)
+                return _updatetonext(ui, repo, repo[choosedrev], display, opts)
         else:
             if not opts['evolve'] or not aspchildren:
                 if filtered:
@@ -1071,7 +1068,7 @@ def cmdnext(ui, repo, **opts):
                 if choosedrev is None:
                     ui.warn(_(b"ambiguous next (unstable) changeset:\n"))
                     for c in aspchildren:
-                        displayer.show(repo[c])
+                        display(repo[c])
                     ui.warn(_(b"(run 'hg evolve --rev REV' on one of them)\n"))
                     return 1
                 else:
@@ -1102,7 +1099,7 @@ def _nextevolve(ui, repo, aspchildren, opts):
                   % ui.label(bytes(repo[b'.']), b'evolve.node'))
     return 0
 
-def _updatetonext(ui, repo, child, displayer, opts):
+def _updatetonext(ui, repo, child, display, opts):
     """ logic for `hg next` command to update to children and move bookmarks if
     required """
     bm = repo._activebookmark
@@ -1139,7 +1136,7 @@ def _updatetonext(ui, repo, child, displayer, opts):
             finally:
                 lockmod.release(tr, lock)
     if not ui.quiet:
-        displayer.show(child)
+        display(child)
     return 0
 
 @eh.wrapcommand(b'commit')
@@ -1179,31 +1176,6 @@ def commitwrapper(orig, ui, repo, *arg, **kwargs):
     finally:
         lockmod.release(tr, lock, wlock)
 
-@eh.wrapcommand(b'strip', extension=b'strip', opts=[
-    (b'', b'bundle', None, _(b"delete the commit entirely and move it to a "
-                             b"backup bundle")),
-    ])
-def stripwrapper(orig, ui, repo, *revs, **kwargs):
-    if (not ui.configbool(b'experimental', b'prunestrip')
-        or kwargs.get('bundle', False)):
-        return orig(ui, repo, *revs, **kwargs)
-
-    if kwargs.get('force'):
-        ui.warn(_(b"warning: --force has no effect during strip with evolve "
-                  b"enabled\n"))
-    if kwargs.get('no_backup', False):
-        ui.warn(_(b"warning: --no-backup has no effect during strips with "
-                  b"evolve enabled\n"))
-
-    revs = list(revs) + kwargs.pop('rev', [])
-    revs = set(scmutil.revrange(repo, revs))
-    revs = repo.revs(b"(%ld)::", revs)
-    kwargs['rev'] = []
-    kwargs['new'] = []
-    kwargs['successor'] = []
-    kwargs['biject'] = False
-    return cmdrewrite.cmdprune(ui, repo, *revs, **kwargs)
-
 @eh.extsetup
 def oldevolveextsetup(ui):
     entry = cmdutil.findcmd(b'commit', commands.table)[1]
@@ -1220,7 +1192,7 @@ def _checkinvalidmarkers(orig, markers):
     if r'debugobsconvert' in sys.argv:
         return
     for mark in markers:
-        if node.nullid in mark[1]:
+        if nullid in mark[1]:
             msg = _(b'bad obsolescence marker detected: invalid successors nullid')
             hint = _(b'You should run `hg debugobsconvert`')
             raise error.Abort(msg, hint=hint)
