@@ -22,7 +22,6 @@ from mercurial import (
     error,
     hg,
     lock as lockmod,
-    logcmdutil,
     merge,
     node,
     obsolete,
@@ -1334,9 +1333,9 @@ def touch(ui, repo, *revs, **opts):
         with ui.configoverride(overrides, b'touch'):
             rewriteutil.precheck(repo, revs, b'touch')
     tmpl = utility.shorttemplate
-    displayer = logcmdutil.changesetdisplayer(ui, repo, {b'template': tmpl})
+    display = compat.format_changeset_summary_fn(ui, repo, b'touch', tmpl)
     with repo.wlock(), repo.lock(), repo.transaction(b'touch'):
-        touchnodes(ui, repo, revs, displayer, **opts)
+        touchnodes(ui, repo, revs, display, **opts)
 
 def touchnodes(ui, repo, revs, displayer, **opts):
     duplicate = opts['duplicate']
@@ -1355,7 +1354,7 @@ def touchnodes(ui, repo, revs, displayer, **opts):
 
         if not (duplicate or allowdivergence):
             if precheck_contentdiv(repo, ctx):
-                displayer.show(ctx)
+                displayer(ctx)
                 index = ui.promptchoice(
                     _(b"reviving this changeset will create divergence"
                       b" unless you make a duplicate.\n(a)llow divergence or"
