@@ -19,7 +19,6 @@ from mercurial import (
     encoding,
     error,
     hg,
-    logcmdutil,
     merge,
     mergeutil,
     node as nodemod,
@@ -67,27 +66,26 @@ def _solveone(ui, repo, ctx, evolvestate, dryrun, confirm,
     """
     tr = repo.currenttransaction()
     assert tr is not None
-    displayer = None
     template = shorttemplate
     if stacktmplt:
         template = stacktemplate
-    displayer = logcmdutil.changesetdisplayer(ui, repo,
-                                              {b'template': template})
+    display = compat.format_changeset_summary_fn(ui, repo, b'evolve',
+                                                 template)
     if b'orphan' == category:
-        result = _solveorphan(ui, repo, ctx, evolvestate, displayer,
+        result = _solveorphan(ui, repo, ctx, evolvestate, display,
                               dryrun, confirm, progresscb)
     elif b'phasedivergent' == category:
         result = _solvephasedivergence(ui, repo, ctx, evolvestate,
-                                       displayer, dryrun, confirm,
+                                       display, dryrun, confirm,
                                        progresscb)
     elif b'contentdivergent' == category:
-        result = _solvedivergent(ui, repo, ctx, evolvestate, displayer,
+        result = _solvedivergent(ui, repo, ctx, evolvestate, display,
                                  dryrun, confirm, progresscb)
     else:
         assert False, b"unknown trouble category: %s" % (category)
     return result
 
-def _solveorphan(ui, repo, orig, evolvestate, displayer, dryrun=False,
+def _solveorphan(ui, repo, orig, evolvestate, display, dryrun=False,
                  confirm=False, progresscb=None):
     """ Tries to stabilize the changeset orig which is orphan.
 
@@ -145,13 +143,13 @@ def _solveorphan(ui, repo, orig, evolvestate, displayer, dryrun=False,
     target = repo[newer]
     if not ui.quiet or confirm:
         repo.ui.write(_(b'move:'), label=b'evolve.operation')
-        displayer.show(orig)
+        display(orig)
         # lastsolved: keep track of successor of last troubled cset we evolved
         # to confirm that if atop msg should be suppressed to remove redundancy
         lastsolved = evolvestate.get(b'lastsolved')
         if lastsolved is None or target != repo[lastsolved]:
             repo.ui.write(_(b'atop:'))
-            displayer.show(target)
+            display(target)
     if confirm and ui.prompt(b'perform evolve? [Ny]', b'n') != b'y':
         raise error.Abort(_(b'evolve aborted by user'))
     todo = b'hg rebase -r %s -d %s\n' % (orig, target)
@@ -169,7 +167,7 @@ def _solveorphan(ui, repo, orig, evolvestate, displayer, dryrun=False,
                               keepbranch, b'orphan')
             return (True, newid)
 
-def _solvephasedivergence(ui, repo, bumped, evolvestate, displayer,
+def _solvephasedivergence(ui, repo, bumped, evolvestate, display,
                           dryrun=False, confirm=False, progresscb=None):
     """Stabilize a phase divergent changeset
 
@@ -196,9 +194,9 @@ def _solvephasedivergence(ui, repo, bumped, evolvestate, displayer,
 
     if not ui.quiet or confirm:
         repo.ui.write(_(b'recreate:'), label=b'evolve.operation')
-        displayer.show(bumped)
+        display(bumped)
         repo.ui.write(_(b'atop:'))
-        displayer.show(prec)
+        display(prec)
     if confirm and ui.prompt(_(b'perform evolve? [Ny]'), b'n') != b'y':
         raise error.Abort(_(b'evolve aborted by user'))
     if dryrun:
@@ -286,9 +284,9 @@ def _resolvephasedivergent(ui, repo, prec, bumped):
         repo.dirstate.setparents(newid, nodemod.nullid)
     return (True, replacementnode)
 
-def _pickresolutionparent(ui, repo, divergent, other, evolvestate):
-    """ if relocation required, decide which divergent cset will be relocated
-    to the other side"""
+def _pickresolutionparent(ui, repo, divergent, other, base, evolvestate):
+    """When the divergent changeset are not based on the same parent, we need
+    to find out where the divergence resolution will be based."""
 
     # we don't handle merge content-divergent changesets yet
     if len(other.parents()) > 1:
@@ -312,12 +310,13 @@ def _pickresolutionparent(ui, repo, divergent, other, evolvestate):
         # Otherwise, we are going to rebase the "behind" branch up to the new
         # brancmap level.
         publicdiv = divergent if other.mutable() else other
-        resolutionparent = publicdiv.p1().node()
+        resolutionparent = publicdiv.p1().rev()
         evolvestate[b'public-divergent'] = publicdiv.node()
         return (True, resolutionparent)
 
     otherp1 = succsotherp1 = other.p1().rev()
     divp1 = succsdivp1 = divergent.p1().rev()
+    basep1 = base.p1().rev()
 
     # finding single successors of divp1 and otherp1
     try:
@@ -334,19 +333,16 @@ def _pickresolutionparent(ui, repo, divergent, other, evolvestate):
                 b"%s\n") % (divergent, other)
         ui.write_err(msg)
         return (False, b".")
+    try:
+        utility._singlesuccessor(repo, base.p1())
+    except utility.MultipleSuccessorsError:
+        msg = (b"ambiguous orphan resolution parent for "
+               b"%s (base)\n") % base
+        ui.debug(msg)
 
     # the changeset on which resolution changeset will be based on
-    resolutionparent = repo[succsdivp1].node()
+    resolutionparent = succsdivp1
 
-    # the nullrev has to be handled specially because -1 is overloaded to both
-    # mean nullrev (this meaning is used for the result of changectx.rev(), as
-    # called above) and the tipmost revision (this meaning is used for the %d
-    # format specifier, as used below)
-    if nodemod.nullrev in (succsotherp1, succsdivp1):
-        # nullrev is the only possible ancestor if succsotherp1 or succsdivp1 is nullrev
-        gca = [nodemod.nullrev]
-    else:
-        gca = repo.revs(b"ancestor(%d, %d)" % (succsotherp1, succsdivp1))
     # divonly: non-obsolete csets which are topological ancestor of "divergent"
     # but not "other"
     divonly = repo.revs(b"only(%d, %d) - obsolete()" % (divergent.rev(),
@@ -378,39 +374,55 @@ def _pickresolutionparent(ui, repo, divergent, other, evolvestate):
     elif divergent == other.p1():
         # both are in parent-child relation
         pass
-    elif succsotherp1 in gca and succsdivp1 in gca:
-        # both are not on the same parent but have same parents's succs.
-        if otheronly and divonly:
-            # case: we have visible csets on both side diverging from
-            # tca of "divergent" and "other". We still need to decide what
-            # to do in this case
-            pass
-        if otheronly:
-            resolutionparent = repo[succsotherp1].node()
-        elif divonly:
-            pass
+    elif basep1 in (divp1, otherp1):
+        # only one side moved, set parent of moved one as resolution parent
+        if basep1 == divp1:
+            resolutionparent = succsotherp1
         else:
-            # no extra cset on either side
             pass
-    elif succsotherp1 in gca and succsdivp1 not in gca:
-        pass
-    elif succsdivp1 in gca and succsotherp1 not in gca:
-        resolutionparent = repo[succsotherp1].node()
     else:
-        msg = _(b"skipping %s: have a different parent than %s "
-                b"(not handled yet)\n") % (divergent, other)
-        hint = _(b"| %(d)s, %(o)s are not based on the same changeset.\n"
-                 b"| With the current state of its implementation,\n"
-                 b"| evolve does not work in that case.\n"
-                 b"| rebase one of them next to the other and run\n"
-                 b"| this command again.\n"
-                 b"| - either: hg rebase --dest 'p1(%(d)s)' -r %(o)s\n"
-                 b"| - or:     hg rebase --dest 'p1(%(o)s)' -r %(d)s\n"
-                 ) % {b'd': divergent, b'o': other}
-        ui.write_err(msg)
-        ui.write_err(hint)
-        return (False, b".")
+        # the nullrev has to be handled specially because -1 is overloaded to both
+        # mean nullrev (this meaning is used for the result of changectx.rev(), as
+        # called above) and the tipmost revision (this meaning is used for the %d
+        # format specifier, as used below)
+        if nodemod.nullrev in (succsotherp1, succsdivp1):
+            # nullrev is the only possible ancestor if succsotherp1 or succsdivp1 is nullrev
+            gca = [nodemod.nullrev]
+        else:
+            gca = repo.revs(b"ancestor(%d, %d)" % (succsotherp1, succsdivp1))
 
+        if succsotherp1 in gca and succsdivp1 in gca:
+            # both are not on the same parent but have same parents's succs.
+            if otheronly and divonly:
+                # case: we have visible csets on both side diverging from
+                # tca of "divergent" and "other". We still need to decide what
+                # to do in this case
+                pass
+            if otheronly:
+                resolutionparent = succsotherp1
+            elif divonly:
+                pass
+            else:
+                # no extra cset on either side
+                pass
+        elif succsotherp1 in gca and succsdivp1 not in gca:
+            pass
+        elif succsdivp1 in gca and succsotherp1 not in gca:
+            resolutionparent = succsotherp1
+        else:
+            msg = _(b"skipping %s: have a different parent than %s "
+                    b"(not handled yet)\n") % (divergent, other)
+            hint = _(b"| %(d)s, %(o)s are not based on the same changeset.\n"
+                     b"| With the current state of its implementation,\n"
+                     b"| evolve does not work in that case.\n"
+                     b"| rebase one of them next to the other and run\n"
+                     b"| this command again.\n"
+                     b"| - either: hg rebase --dest 'p1(%(d)s)' -r %(o)s\n"
+                     b"| - or:     hg rebase --dest 'p1(%(o)s)' -r %(d)s\n"
+                    ) % {b'd': divergent, b'o': other}
+            ui.write_err(msg)
+            ui.write_err(hint)
+            return (False, b".")
     return (True, resolutionparent)
 
 def _relocatedivergent(repo, orig, dest, evolvestate):
@@ -421,7 +433,7 @@ def _relocatedivergent(repo, orig, dest, evolvestate):
     with state.saver(evolvestate, {b'current': orig.node()}), configoverride:
         return _relocate(repo, orig, dest, evolvestate, keepbranch=True)
 
-def _solvedivergent(ui, repo, divergent, evolvestate, displayer, dryrun=False,
+def _solvedivergent(ui, repo, divergent, evolvestate, display, dryrun=False,
                     confirm=False, progresscb=None):
     """tries to solve content-divergence of a changeset
 
@@ -474,8 +486,9 @@ def _solvedivergent(ui, repo, divergent, evolvestate, displayer, dryrun=False,
     evolvestate[b'old-divergent'] = None
 
     # setup everything before merging two content-divergent csets
-    picked, resolutionparent = _pickresolutionparent(ui, repo, divergent,
-                                                     other, evolvestate)
+    picked, resolutionparent = _pickresolutionparent(ui, repo, divergent, other,
+                                                     base, evolvestate)
+    resolutionparent = repo[resolutionparent].node()
     if not picked:
         return (False, b".")
     evolvestate[b'resolutionparent'] = resolutionparent
@@ -487,11 +500,11 @@ def _solvedivergent(ui, repo, divergent, evolvestate, displayer, dryrun=False,
 
     if not ui.quiet or confirm:
         ui.write(_(b'merge:'), label=b'evolve.operation')
-        displayer.show(divergent)
+        display(divergent)
         ui.write(_(b'with: '))
-        displayer.show(other)
+        display(other)
         ui.write(_(b'base: '))
-        displayer.show(base)
+        display(base)
     if confirm and ui.prompt(_(b'perform evolve? [Ny]'), b'n') != b'y':
         raise error.Abort(_(b'evolve aborted by user'))
     if dryrun:
@@ -958,8 +971,6 @@ def _relocatecommit(repo, orig, dest, pctx, keepbranch, commitmsg):
         raise error.InterventionRequired(_(b"unresolved merge conflicts"),
                                          hint=hint)
 
-    if commitmsg is None:
-        commitmsg = orig.description()
     extra = dict(orig.extra())
     if b'branch' in extra:
         del extra[b'branch']
@@ -1592,7 +1603,7 @@ def _performevolve(ui, repo, **opts):
         if evolvestate[b'command'] != b'evolve':
             evolvestate.delete()
             return
-        startnode = repo.unfiltered()[evolvestate[b'startnode']]
+        startnode = evolvestate[b'startnode']
         if b'update' in evolvestate:
             shouldupdate = evolvestate[b'update']
         evolvestate.delete()
@@ -1714,8 +1725,8 @@ def solveobswdp(ui, repo, opts):
     oldid = repo[b'.'].node()
     startctx = repo[b'.']
     dryrunopt = opts.get('dry_run', False)
-    displayer = logcmdutil.changesetdisplayer(ui, repo,
-                                              {b'template': shorttemplate})
+    display = compat.format_changeset_summary_fn(ui, repo, b'evolve',
+                                                 shorttemplate)
     try:
         ctx = repo[utility._singlesuccessor(repo, repo[b'.'])]
     except utility.MultipleSuccessorsError as exc:
@@ -1723,12 +1734,12 @@ def solveobswdp(ui, repo, opts):
                             b' successors:\n'))
         for ln in exc.successorssets:
             for n in ln:
-                displayer.show(repo[n])
+                display(repo[n])
         return 2
 
     ui.status(_(b'update:'))
     if not ui.quiet:
-        displayer.show(ctx)
+        display(ctx)
 
     if dryrunopt:
         return 0
