@@ -61,7 +61,7 @@ For merge revision, we reuse as much as possible of the parents order:
     pl = stablemin(parents(m))
     ph = stablemax(parents(m))
     stablesort(::m) == stablesort(pl)
-                       + [i for in in stablesort(ph) if in ph % pl]
+                       + [i for i in stablesort(ph) if i in ph % pl]
                        + m
 
 This is visible in the example above:
@@ -113,7 +113,7 @@ The stable order of `stablesort(::H)` match our definition::
   stablesort(::F) = [A, B, C, E, F]
   stablesort(::G) - ::F = [A, B, C, D, E, G] - [A, B, C, E, F] = [D, G]
 
-In this order, we reuse all of `stablesort(::H)`, but the subset of
+In this order, we reuse all of `stablesort(::F)`, but the subset of
 `stablesort(::G)` we reuse is not contiguous, we had to skip over 'E' that is
 already contained inside ::F.
 
