@@ -232,9 +232,9 @@ colortable = {b'topic.active': b'green',
               b'topic.active': b'green',
               }
 
-__version__ = b'0.20.0'
+__version__ = b'0.21.0'
 
-testedwith = b'4.6.2 4.7 4.8 4.9 5.0 5.1 5.2 5.3 5.4 5.5 5.6'
+testedwith = b'4.6.2 4.7 4.8 4.9 5.0 5.1 5.2 5.3 5.4 5.5 5.6 5.7'
 minimumhgversion = b'4.6'
 buglink = b'https://bz.mercurial-scm.org/'
 
@@ -294,7 +294,16 @@ if util.safehasattr(registrar, 'configitem'):
 def _contexttopic(self, force=False):
     if not (force or self.mutable()):
         return b''
-    return self.extra().get(constants.extrakey, b'')
+    cache = getattr(self._repo, '_topiccache', None)
+    # topic loaded, but not enabled (eg: multiple repo in the same process)
+    if cache is None:
+        return b''
+    topic = cache.get(self.rev())
+    if topic is None:
+        topic = self.extra().get(constants.extrakey, b'')
+        self._repo._topiccache[self.rev()] = topic
+    return topic
+
 context.basectx.topic = _contexttopic
 
 def _contexttopicidx(self):
@@ -362,6 +371,10 @@ def _nodemap(repo, node):
         return [t]
     return []
 
+def wrap_summary(orig, ui, repo, *args, **kwargs):
+    with discovery.override_context_branch(repo) as repo:
+        return orig(ui, repo, *args, **kwargs)
+
 def uisetup(ui):
     destination.modsetup(ui)
     discovery.modsetup(ui)
@@ -397,6 +410,8 @@ def uisetup(ui):
     # plug into logic for this into mergemod.update().
     extensions.wrapcommand(commands.table, b'update', checkt0)
 
+    extensions.wrapcommand(commands.table, b'summary', wrap_summary)
+
     try:
         evolve = extensions.find(b'evolve')
         extensions.wrapfunction(evolve.rewriteutil, "presplitupdate",
@@ -435,6 +450,12 @@ def reposetup(ui, repo):
         repo.ui.setconfig(b'experimental', b'thg.displaynames', b'topics',
                           source=b'topic-extension')
 
+    # BUG: inmemory rebase drops the topic, and fails to switch to the new
+    # topic.  Disable inmemory rebase for now.
+    if repo.ui.configbool(b'rebase', b'experimental.inmemory'):
+        repo.ui.setconfig(b'rebase', b'experimental.inmemory', b'False',
+                          source=b'topic-extension')
+
     class topicrepo(repo.__class__):
 
         # attribute for other code to distinct between repo with topic and repo without
@@ -481,6 +502,10 @@ def reposetup(ui, repo):
                 del ctx.extra()[constants.extrakey]
             return super(topicrepo, self).commitctx(ctx, *args, **kwargs)
 
+        @util.propertycache
+        def _topiccache(self):
+            return {}
+
         @property
         def topics(self):
             if self._topics is not None:
@@ -510,8 +535,8 @@ def reposetup(ui, repo):
         def branchheads(self, branch=None, start=None, closed=False):
             if branch is None:
                 branch = self[None].branch()
-            if self.currenttopic:
-                branch = b"%s:%s" % (branch, self.currenttopic)
+                if self.currenttopic:
+                    branch = b"%s:%s" % (branch, self.currenttopic)
             return super(topicrepo, self).branchheads(branch=branch,
                                                       start=start,
                                                       closed=closed)
@@ -1009,7 +1034,7 @@ def _applyconvertbmarktopic(ui, repo, revs, old, bmark, tr):
         return
     ui.status(_(b'changed topic to "%s" on %d revisions\n') % (bmark,
               rewrote))
-    ui.debug(b'removing bookmark "%s" from "%d"' % (bmark, old))
+    ui.debug(b'removing bookmark "%s" from "%d"\n' % (bmark, old))
     bookmarks.delete(repo, tr, [bmark])
 
 def _changecurrenttopic(repo, newtopic):
